@@ -1,20 +1,10 @@
-// Copyright 2017 The Rust Project Developers. See the COPYRIGHT
-// file at the top-level directory of this distribution and at
-// http://rust-lang.org/COPYRIGHT.
-//
-// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
-// http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
-// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
-// option. This file may not be copied, modified, or distributed
-// except according to those terms.
-
 //! The data that we will serialize and deserialize.
 
-use dep_graph::DepNode;
-use ich::Fingerprint;
-use rustc_data_structures::indexed_vec::{IndexVec, Idx};
+use crate::dep_graph::DepNode;
+use crate::ich::Fingerprint;
+use rustc_index::vec::{IndexVec, Idx};
 
-newtype_index! {
+rustc_index::newtype_index! {
     pub struct SerializedDepNodeIndex { .. }
 }
 
