@@ -1,13 +1,13 @@
 use rustc::util::nodemap::DefIdSet;
 
 use crate::clean;
-use crate::fold::{DocFolder};
 use crate::core::DocContext;
-use crate::passes::{ImplStripper, ImportStripper, Stripper, Pass};
+use crate::fold::DocFolder;
+use crate::passes::{ImplStripper, ImportStripper, Pass, Stripper};
 
 pub const STRIP_PRIVATE: Pass = Pass {
     name: "strip-private",
-    pass: strip_private,
+    run: strip_private,
     description: "strips all private items from a crate which cannot be seen externally, \
         implies strip-priv-imports",
 };
