@@ -1,14 +1,3 @@
-// Copyright 2016 The Rust Project Developers. See the COPYRIGHT
-// file at the top-level directory of this distribution and at
-// http://rust-lang.org/COPYRIGHT.
-//
-// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
-// http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
-// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
-// option. This file may not be copied, modified, or distributed
-// except according to those terms.
-
-
 // This test case tests the incremental compilation hash (ICH) implementation
 // for statics.
 
@@ -16,7 +5,7 @@
 // and make sure that the hash has changed, then change nothing between rev2 and
 // rev3 and make sure that the hash has not changed.
 
-// compile-pass
+// build-pass (FIXME(62277): could be check-pass?)
 // revisions: cfail1 cfail2 cfail3
 // compile-flags: -Z query-dep-graph -Zincremental-ignore-spans
 
@@ -27,7 +16,7 @@
 #![crate_type="rlib"]
 
 
-// Change static visibility ---------------------------------------------------
+// Change static visibility
 #[cfg(cfail1)]
 static STATIC_VISIBILITY: u8 = 0;
 
@@ -37,7 +26,7 @@ static STATIC_VISIBILITY: u8 = 0;
 pub static STATIC_VISIBILITY: u8 = 0;
 
 
-// Change static mutability ---------------------------------------------------
+// Change static mutability
 #[cfg(cfail1)]
 static STATIC_MUTABILITY: u8 = 0;
 
@@ -47,7 +36,7 @@ static STATIC_MUTABILITY: u8 = 0;
 static mut STATIC_MUTABILITY: u8 = 0;
 
 
-// Add linkage attribute ------------------------------------------------------
+// Add linkage attribute
 #[cfg(cfail1)]
 static STATIC_LINKAGE: u8 = 0;
 
@@ -58,7 +47,7 @@ static STATIC_LINKAGE: u8 = 0;
 static STATIC_LINKAGE: u8 = 0;
 
 
-// Add no_mangle attribute ----------------------------------------------------
+// Add no_mangle attribute
 #[cfg(cfail1)]
 static STATIC_NO_MANGLE: u8 = 0;
 
@@ -69,7 +58,7 @@ static STATIC_NO_MANGLE: u8 = 0;
 static STATIC_NO_MANGLE: u8 = 0;
 
 
-// Add thread_local attribute -------------------------------------------------
+// Add thread_local attribute
 #[cfg(cfail1)]
 static STATIC_THREAD_LOCAL: u8 = 0;
 
@@ -80,27 +69,27 @@ static STATIC_THREAD_LOCAL: u8 = 0;
 static STATIC_THREAD_LOCAL: u8 = 0;
 
 
-// Change type from i16 to u64 ------------------------------------------------
+// Change type from i16 to u64
 #[cfg(cfail1)]
 static STATIC_CHANGE_TYPE_1: i16 = 0;
 
 #[cfg(not(cfail1))]
-#[rustc_clean(cfg="cfail2", except="Hir,HirBody,TypeOfItem")]
+#[rustc_clean(cfg="cfail2", except="Hir,HirBody,type_of")]
 #[rustc_clean(cfg="cfail3")]
 static STATIC_CHANGE_TYPE_1: u64 = 0;
 
 
-// Change type from Option<i8> to Option<u16> ---------------------------------
+// Change type from Option<i8> to Option<u16>
 #[cfg(cfail1)]
 static STATIC_CHANGE_TYPE_2: Option<i8> = None;
 
 #[cfg(not(cfail1))]
-#[rustc_clean(cfg="cfail2", except="Hir,HirBody,TypeOfItem")]
+#[rustc_clean(cfg="cfail2", except="Hir,HirBody,type_of")]
 #[rustc_clean(cfg="cfail3")]
 static STATIC_CHANGE_TYPE_2: Option<u16> = None;
 
 
-// Change value between simple literals ---------------------------------------
+// Change value between simple literals
 #[rustc_clean(cfg="cfail2", except="HirBody")]
 #[rustc_clean(cfg="cfail3")]
 static STATIC_CHANGE_VALUE_1: i16 = {
@@ -112,7 +101,7 @@ static STATIC_CHANGE_VALUE_1: i16 = {
 };
 
 
-// Change value between expressions -------------------------------------------
+// Change value between expressions
 #[rustc_clean(cfg="cfail2", except="HirBody")]
 #[rustc_clean(cfg="cfail3")]
 static STATIC_CHANGE_VALUE_2: i16 = {
@@ -144,7 +133,7 @@ static STATIC_CHANGE_VALUE_4: i16 = {
 };
 
 
-// Change type indirectly -----------------------------------------------------
+// Change type indirectly
 struct ReferencedType1;
 struct ReferencedType2;
 
@@ -155,11 +144,11 @@ mod static_change_type_indirectly {
     #[cfg(not(cfail1))]
     use super::ReferencedType2 as Type;
 
-    #[rustc_clean(cfg="cfail2", except="Hir,HirBody,TypeOfItem")]
+    #[rustc_clean(cfg="cfail2", except="Hir,HirBody,type_of")]
     #[rustc_clean(cfg="cfail3")]
     static STATIC_CHANGE_TYPE_INDIRECTLY_1: Type = Type;
 
-    #[rustc_clean(cfg="cfail2", except="Hir,HirBody,TypeOfItem")]
+    #[rustc_clean(cfg="cfail2", except="Hir,HirBody,type_of")]
     #[rustc_clean(cfg="cfail3")]
     static STATIC_CHANGE_TYPE_INDIRECTLY_2: Option<Type> = None;
 }
