@@ -5,10 +5,9 @@ fn main() {
     let _: [u8; 0] = [4; {
         match &1 as *const i32 as usize {
             //~^ ERROR casting pointers to integers in constants
-            //~| NOTE for more information, see
-            0 => 42, //~ ERROR constant contains unimplemented expression type
-            //~^ NOTE "pointer arithmetic or comparison" needs an rfc before being allowed
+            //~| ERROR `match` is not allowed in a `const`
             //~| ERROR evaluation of constant value failed
+            0 => 42,
             n => n,
         }
     }];
