@@ -16,7 +16,7 @@ struct Foo<T> {
 #[rustc_if_this_changed(Krate)]
 type TypeAlias<T> = Foo<T>;
 
-#[rustc_then_this_would_need(ItemVariances)] //~ ERROR OK
+#[rustc_then_this_would_need(variances_of)] //~ ERROR OK
 struct Use<T> {
     x: TypeAlias<T>
 }
