@@ -1,13 +1,3 @@
-// Copyright 2015 The Rust Project Developers. See the COPYRIGHT
-// file at the top-level directory of this distribution and at
-// http://rust-lang.org/COPYRIGHT.
-//
-// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
-// http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
-// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
-// option. This file may not be copied, modified, or distributed
-// except according to those terms.
-
 // Test use of advanced const fn without the `const_fn` feature gate.
 
 const fn foo() -> usize { 0 } // ok
@@ -19,14 +9,16 @@ trait Foo {
                                 //~| ERROR trait fns cannot be declared const
 }
 
-impl Foo {
-    const fn baz() -> u32 { 0 } // ok
-}
-
 impl Foo for u32 {
     const fn foo() -> u32 { 0 } //~ ERROR trait fns cannot be declared const
 }
 
+trait Bar {}
+
+impl dyn Bar {
+    const fn baz() -> u32 { 0 } // ok
+}
+
 static FOO: usize = foo();
 const BAR: usize = foo();
 
