@@ -1,17 +1,8 @@
-// Copyright 2017 The Rust Project Developers. See the COPYRIGHT
-// file at the top-level directory of this distribution and at
-// http://rust-lang.org/COPYRIGHT.
-//
-// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
-// http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
-// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
-// option. This file may not be copied, modified, or distributed
-// except according to those terms.
-
 #![feature(generators, generator_trait)]
 
 use std::ops::{GeneratorState, Generator};
 use std::cell::Cell;
+use std::pin::Pin;
 
 fn borrow_local_inline() {
     // Not OK to yield with a borrow of a temporary.
@@ -19,22 +10,23 @@ fn borrow_local_inline() {
     // (This error occurs because the region shows up in the type of
     // `b` and gets extended by region inference.)
     let mut b = move || {
-        let a = &3;
+        let a = &mut 3;
+        //~^ ERROR borrow may still be in use when generator yields
         yield();
         println!("{}", a);
     };
-    b.resume();
+    Pin::new(&mut b).resume();
 }
 
 fn borrow_local_inline_done() {
     // No error here -- `a` is not in scope at the point of `yield`.
     let mut b = move || {
         {
-            let a = &3;
+            let a = &mut 3;
         }
         yield();
     };
-    b.resume();
+    Pin::new(&mut b).resume();
 }
 
 fn borrow_local() {
@@ -45,12 +37,13 @@ fn borrow_local() {
     let mut b = move || {
         let a = 3;
         {
-            let b = &a; //~ ERROR
+            let b = &a;
+            //~^ ERROR borrow may still be in use when generator yields
             yield();
             println!("{}", b);
         }
     };
-    b.resume();
+    Pin::new(&mut b).resume();
 }
 
 fn main() { }
