@@ -1,10 +1,9 @@
 // Issue #876
 
-#![no_implicit_prelude]
 use std::vec::Vec;
 
 fn last<T>(v: Vec<&T> ) -> std::option::Option<T> {
-    panic!();
+    ::std::panic!();
 }
 
 fn main() {
@@ -12,6 +11,6 @@ fn main() {
     let x : char = last(y);
     //~^ ERROR mismatched types
     //~| expected type `char`
-    //~| found type `std::option::Option<_>`
-    //~| expected char, found enum `std::option::Option`
+    //~| found enum `std::option::Option<_>`
+    //~| expected `char`, found enum `std::option::Option`
 }
